/*
  Take Care of Your Plants
  Plants bring light and color into our house but unfortunately it is not so easy to take care of them.
  One needs to know how to grow them and give them the proper care so that they can grow over time.
  We wanted to create a system that would help us give our plants the care they deserve.
  Our system supports the following scenarios:
  1. The CPX measures the moisture level in the plant and displays these values ​​in the Blynk app continuously.
  If the moisture values ​​of the plant are too high or too low, an appropriate indication will appear in the app 
  and we will know how to water the plant according to its moisture value.
  2. In order to prevent damage to the leaves of the plant (by pests or insects),in case of an abnormal movement
  around the plant the CPX will detect this and warn this in a mobile message. In addition, the CPX will beep to move the pests away.
  3. In order to make sure that we do not forget to water the plant,every two days (or another set time frame that is set)
  a watering task is automatically created in Google Tasks. In addition the CPX neopixels turn on.
  4. Clicking the right or left button on the CPX will complete the watering task on Google Tasks and turn off the neopixels.
  According to the button that was pressed the Blynk app will present the last person who watered the plant.
  5. To encourage the growth of the plant, you can activate a melody from the Blynk app that the CPX will play to the plant.
     
  Video link: https://www.youtube.com/watch?v=R0YjBEoyFWo
*/

#define BLYNK_AUTH_TOKEN //your Blynk Auth Token
#define BLYNK_PRINT SerialUSB
#define BLYNK_DEBUG SerialUSB

#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <Adafruit_CircuitPlayground.h>
#include "pitches.h"

char auth[] = BLYNK_AUTH_TOKEN;
char ssid[] = //your wifi network name;
char pass[] = //your wifi network password;

#define EspSerial Serial1
#define ESP8266_BAUD 115200
#define LOW_CAP_THRESHOLD 700
#define HIGH_CAP_THRESHOLD 1100

int currentHumidity;
const int speed = 2;
const int numNotes = 26;
boolean rightButtonPressed = CircuitPlayground.rightButton();
boolean leftButtonPressed = CircuitPlayground.leftButton();
int melody[] = {                          
  NOTE_B4, NOTE_B4, NOTE_B4, 
  NOTE_B4, NOTE_B4, NOTE_B4,
  NOTE_B4, NOTE_D5, NOTE_G4, NOTE_A4,
  NOTE_B4, 0,
  NOTE_C5, NOTE_C5, NOTE_C5, NOTE_C5, 
  NOTE_C5, NOTE_B4, NOTE_B4, NOTE_B4, NOTE_B4, 
  NOTE_D5, NOTE_D5, NOTE_C5, NOTE_A4, 
  NOTE_G4};
int noteDurations[] = {    
  4, 4, 2, 
  4, 4, 2, 
  4, 4, 3, 8,
  1, 4,
  4, 4, 3, 8, 
  4, 4, 4, 8, 8,
  4, 4, 4, 4, 
  1
  };

ESP8266 wifi(&EspSerial);

void setup() {
  CircuitPlayground.begin();
  SerialUSB.begin(115200);
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Blynk.begin(auth, wifi, ssid, pass);
  Serial.begin(115200);
  Blynk.setProperty(V4, "label", "The last person who watered the plants:");
  Blynk.virtualWrite(V4, " ");
}

void loop() {
  Blynk.run(); 
  checkHumidity();
  movementAlert(); 
}

void checkHumidity(){
  currentHumidity = CircuitPlayground.readCap(A3); 
  if (currentHumidity < LOW_CAP_THRESHOLD || currentHumidity > HIGH_CAP_THRESHOLD) {
    Blynk.setProperty(V0, "color", "#D3435C"); //RED
  } else{
    Blynk.setProperty(V0, "color", "#04C0F8"); //BLUE
  }
  Blynk.virtualWrite(V0, currentHumidity);
}

void movementAlert(){
  if (CircuitPlayground.motionZ() < 0){
    Blynk.virtualWrite(V1, "Note that unusual movement was detected in the plant");
    for (int i = 0; i < 5; i++){
      CircuitPlayground.playTone(250, 100);
      delay(100);
    }
  }
}

BLYNK_WRITE(V2){
  wateringAlert(); 
}

BLYNK_WRITE(V3){
  playMelody();
}

void playMelody(){
  for (int thisNote = 0; thisNote < numNotes; thisNote++) {
    int noteDuration = (1000 / speed) / noteDurations[thisNote];
    CircuitPlayground.playTone(melody[thisNote], noteDuration);
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
  }
}

void wateringAlert(){
  while(!rightButtonPressed && !leftButtonPressed){
    CircuitPlayground.clearPixels();
    CircuitPlayground.setPixelColor(0, 255, 0, 0);
    CircuitPlayground.setPixelColor(1, 128, 128, 0);
    CircuitPlayground.setPixelColor(2, 0, 255, 0);
    CircuitPlayground.setPixelColor(3, 0, 128, 128);
    CircuitPlayground.setPixelColor(4, 0, 0, 255);
    CircuitPlayground.setPixelColor(5, 0xFF0000);
    CircuitPlayground.setPixelColor(6, 0x808000);
    CircuitPlayground.setPixelColor(7, 0x00FF00);
    CircuitPlayground.setPixelColor(8, 0x008080);
    CircuitPlayground.setPixelColor(9, 0x0000FF); 
    delay(300);
    rightButtonPressed = CircuitPlayground.rightButton();
    leftButtonPressed = CircuitPlayground.leftButton();
  }
  
  CircuitPlayground.clearPixels();
  Blynk.virtualWrite(V1, "The plants were watered");
  if (rightButtonPressed){
    Blynk.setProperty(V4, "color", "#4a2370");
    Blynk.virtualWrite(V4, "Alice");
  } else{
    Blynk.setProperty(V4, "color", "#23C48E");
    Blynk.virtualWrite(V4, "Bob");
  }
  delay(300);
  rightButtonPressed = false;
  leftButtonPressed = false;
}
